import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.stream.Collectors;
import javax.swing.JPanel;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;

public abstract class Chart extends JPanel{

  protected List<Quote> quotes = new ArrayList<>();
  protected String name;
  protected String symbol;
  protected int width = 800;
  protected int height = 600;

  public Chart(String name, String symbol){
    this.name = name;
    this.symbol = symbol;
  }

  public Chart(String name, String symbol, int width, int height){
    this(name, symbol);
    this.width = width;
    this.height = height;
  }

  @Override
  public String toString(){
    return this.name + " " + "[" + this.symbol + "]";
  }

  public void addQuote(String date, double price){
    this.quotes.add(new Quote(date, price));
  }

  public void addQuotes(List<Quote> quotes){
    this.quotes.addAll(quotes);
  }

  public List<Double> getPrices(){
    return this.quotes.stream().map(q -> q.getPrice()).collect(Collectors.toList());
  }

  public Double getMinPrice(){
    return Collections.min(this.getPrices());
  }

  public Double getMaxPrice(){
    return Collections.max(this.getPrices());
  }

  @Override
  public Dimension getPreferredSize(){
    return new Dimension(this.width, this.height);
  }

  @Override
  public void paintComponent(Graphics g){
    super.paintComponent(g);
    this.paintStockPrices(g);
    this.paintStockName(g);
  }

  abstract void paintStockPrices(Graphics g);

  abstract void paintStockName(Graphics g);
}